function [musim,betaysim,betamsim,sigsim,etasim,sstsim,etapost,betapostm,betapostchol,igpost,prob,loglik,perm] = mcmcyasym(y,x,eta,betay,betam,mu,sig,e0,...
                                                                                             prsig,prbem,prbeinf,it0,it);
% 
% function call: [musim,betaysim,betamsim,sigsim,etasim,etapost,prob,loglik] = mcmcyasym(y,x,eta,betay,betam,mu,sig,e0,...
%                                                                                             prsig,prbem,prbeinf,it0,it);
% INPUT:
% y     dependent time series, (nx1) vector
% x     exogenous right hand time series (mon.pol. indicator), (nx1) vector
% eta   initial transition matrix (kxk), rows must be probability vectors
% mu    (kx1) vector, initial values for the constants
% betay initial value for lagged dep. variables (ylagx1)
% betam initial value for the exogenous right-hand variable (mlagxnst)
% sig   (kx1) vector, inital values for the variances
% e0    prior on transition probabilities (kxk)
% prbem prior mean for all regression parameters ([mu;betay;betam(:)])
% prbeinf prior information on all regression parameters ([mu;betay;betam(:)])
% prsig prior on changing variances (kx2)
% it    number of iterations
% it0   number of iterations discarded
%
% OUTPUT:
% simulated values:
% [musim,betaysim,betamsim,sigsim,etasim,etapost,prob,loglik]
%  mu,betay,betam,sigma's,trans.probs.,post.param of the trans.probs,post.stateprobs.,loglikelihood
% 
% Author: Sylvia Kaufmann

rand('state',0);
randn('state',0);
gamrnd('state',0);

perm=2; %switch for constrained permutation (1, set the state-identifying parameter on line 99), random permutation (2)
cf = 0; %switch for computation of bayes factor, 1=candidate's formula (needs constrained permutation)
%																0=bridge sampler
MMC=1000; % number of retained elements to form the mixture q for the bridge sampler

n=size(y,1);
ylag=size(betay,1);
mlag=size(betam,1);
mnst=size(betam,2);
sigsize=size(sig,1);
lags=max([ylag,mlag]);
index=[];for i=1:lags;index=[index;lags-i+1:n-i];end;index=index';
Z=[ones(n-lags,1) y(index(:,1:min([lags,ylag]))) x(index(:,1:min([lags,mlag])))];
y=y(lags+1:end);

nst=size(eta,1);
pini=ones(nst,1)/nst;

n = size(y,1)+1;
states = 1:nst;
ret=500; %number of retained simulated paths for sst
sstsim = zeros(n,ret);%the last ret simulated paths for s_t are retained
etasim=zeros(nst,nst,it);
etapost=zeros(nst,nst,cf*it+(1-cf)*MMC);

betapostm=zeros(ylag+mlag*mnst+nst,(1-cf)*MMC+cf*it);
betapostchol=zeros(ylag+mlag*mnst+nst,ylag+mlag*mnst+nst,(1-cf)*MMC+cf*it);
igpost=zeros(sigsize,2,cf*it+(1-cf)*MMC);

musim=zeros(nst,it);
betaysim=zeros(ylag,it);
betamsim=zeros(mlag,mnst,it);
sigsim=zeros(sigsize,it);
prob(1:nst,1:n) = 0;
loglik=zeros(it,1);

%random selection of elements forming the mixture q for the bridge sampler
indexq = it0+simuni(it-it0,MMC); % selects randomly MMC elements out of L (with repetition)
comp=[1:MMC]';

for j = 1:it
   if any(j==[1:1000:it])
       j
   end
   if sigsize>1;
      [fv sp] = filtvert(y,Z,eta,pini,mu,betay,betam,sig);
   elseif mnst>1;
      [fv sp] = filtvert2(y,Z,eta,pini,mu,betay,betam,sig);
   elseif mnst==1;
      [fv sp] = filtvert3(y,Z,eta,pini,mu,betay,betam,sig);
   end
     if j > 1 loglik(j-1)=sp;
    end;
    sst = simstate(fv,eta);
    [eta cnt] = simtrans(nst,sst(1:end),e0,1.);
    
    if sigsize>1;
    [mu betay betam Zmat par bmean bvar]=simparinfo(y,Z,ylag,mlag,sst,sig,prbem,prbeinf);
    [sig igpar]=simsiginfo(y,Zmat,par,sst,prsig);
    elseif mnst>1;
    [mu betay betam Zmat par bmean bvar]=simparinfo2(y,Z,ylag,mlag,sst,sig,prbem,prbeinf,nst);
    [sig igpar]=simsiginfo2(y,Zmat,par,sst,prsig);
    elseif mnst==1;
    [mu betay betam Zmat par bmean bvar]=simparinfo3(y,Z,ylag,mlag,sst,sig,prbem,prbeinf,nst);
    [sig igpar]=simsiginfo2(y,Zmat,par,sst,prsig);
    end


%  permutation

   if perm==1  %constrained permutation
        [parneu si] = sort(mu); %determine which parameter identifies the state
        [si1,ssort]=sort(si);  
        sst = ssort(sst)';   
        eta = eta(si,si);
        mu=mu(si);
        if mnst>1;
           betam = betam(:,si);
        end
        if sigsize>1;
           sig = sig(si);
        end
    end
     if perm==2  %random permuation
        si = randperm(nst);
        mu=mu(si);
        if sigsize>1;
           sig=sig(si);
           prsig=prsig(si,:);
        end
        if mnst>1;
           betam=betam(:,si);
        end
        %permutation of the prior hyperparameters. Note: the hyperparameters of the prior on mu and betam are not permuted
        % because they are assumed to be state-independent.
        e0=e0(si,si); 
        eta = eta(si,si);
        [si1,ssort]=sort(si);  
        sst = ssort(sst);   
     end   

      etasim(:,:,j) = eta;
      if cf==1;
         etapost(:,:,j)=cnt;
      end
      musim(:,j)=mu;
      betaysim(:,j)=betay;
      betamsim(:,:,j)=betam;
      sigsim(:,j)=sig;
   
     if cf==0
         if any(indexq==j) % MCMC sample wird zum aufbau von q verwendet
            jm=comp(indexq==j); % nummern der componente
            sjm=size(jm,1);
            etapost(:,:,jm)=cnt(:,:,ones(1,sjm));
            betapostm(:,jm)=bmean(:,ones(1,sjm));
            bvarchol=chol(bvar)';
            betapostchol(:,:,jm)=bvarchol(:,:,ones(1,sjm));
            igpost(:,:,jm)=igpar(:,:,ones(1,sjm)); 
         end
      end   

      if j > (it-ret)
          sstsim(:,j-it+ret)=sst';
      end
      if (j > it0)*(perm==1)
         ind = sst(ones(1,nst),:) == states(ones(1,n),:)';
         prob = prob + ind;
      end
   
      
   end
if sigsize>1;
   [fv loglik(it)] = filtvert(y,Z,eta,pini,mu,betay,betam,sig);
elseif mnst>1;
   [fv loglik(it)] = filtvert2(y,Z,eta,pini,mu,betay,betam,sig);
elseif mnst==1;
   [fv loglik(it)] = filtvert3(y,Z,eta,pini,mu,betay,betam,sig);
end
prob = prob/(it-it0);
